@@ -5,8 +5,7 @@
  *   SPDX-License-Identifier: GPL-2.0-only OR GPL-3.0-only OR LicenseRef-KDE-Accepted-GPL
  */
 
-#ifndef FLATPAKBACKEND_H
-#define FLATPAKBACKEND_H
+#pragma once
 
 #include "FlatpakResource.h"
 
@@ -64,7 +63,7 @@ public:
     void addSourceFromFlatpakRepo(const QUrl &url, ResultsStream *stream);
     void addAppFromFlatpakBundle(const QUrl &url, ResultsStream *stream);
     void addAppFromFlatpakRef(const QUrl &url, ResultsStream *stream);
-    FlatpakResource *getAppForInstalledRef(FlatpakInstallation *flatpakInstallation, FlatpakInstalledRef *ref) const;
+    FlatpakResource *getAppForInstalledRef(FlatpakInstallation *flatpakInstallation, FlatpakInstalledRef *ref, bool *freshResource = nullptr) const;
 
     FlatpakSourcesBackend *sources() const
     {
@@ -76,12 +75,23 @@ public:
     void loadRemote(FlatpakInstallation *installation, FlatpakRemote *remote);
     void unloadRemote(FlatpakInstallation *installation, FlatpakRemote *remote);
 
-    HelpfulError *explainDysfunction() const override;
+    InlineMessage *explainDysfunction() const override;
     QVector<FlatpakInstallation *> installations() const
     {
         return m_installations;
     }
 
+    bool isTracked(FlatpakResource *resource) const;
+    QThreadPool *threadPool()
+    {
+        return &m_threadPool;
+    }
+
+    GCancellable *cancellable() const
+    {
+        return m_cancellable;
+    }
+
 private Q_SLOTS:
     void onFetchMetadataFinished(FlatpakResource *resource, const QByteArray &metadata);
     void onFetchSizeFinished(FlatpakResource *resource, guint64 downloadSize, guint64 installedSize);
@@ -92,7 +102,7 @@ Q_SIGNALS: // for tests
 private:
     friend class FlatpakSource;
 
-    void metadataRefreshed();
+    void metadataRefreshed(FlatpakRemote *remote);
     bool flatpakResourceLessThan(AbstractResource *l, AbstractResource *r) const;
     void announceRatingsReady();
     FlatpakInstallation *preferredInstallation() const
@@ -103,6 +113,7 @@ private:
     FlatpakRemote *getFlatpakRemoteByUrl(const QString &url, FlatpakInstallation *installation) const;
     FlatpakResource *getRuntimeForApp(FlatpakResource *resource) const;
     FlatpakResource *resourceForComponent(const AppStream::Component &component, const QSharedPointer<FlatpakSource> &source) const;
+    void checkRepositories(const QMap<QString, QStringList> &names);
 
     void loadAppsFromAppstreamData();
     bool loadAppsFromAppstreamData(FlatpakInstallation *flatpakInstallation);
@@ -119,13 +130,15 @@ private:
 
     QVector<AbstractResource *> resourcesByAppstreamName(const QString &name) const;
     void acquireFetching(bool f);
-    void checkForUpdates(FlatpakInstallation *flatpakInstallation, FlatpakRemote *remote);
+    void checkForRemoteUpdates(FlatpakInstallation *flatpakInstallation, FlatpakRemote *remote);
+    void createPool(QSharedPointer<FlatpakSource> source);
+    FlatpakRemote *installSource(FlatpakResource *resource);
 
     StandardBackendUpdater *m_updater;
     FlatpakSourcesBackend *m_sources = nullptr;
     QSharedPointer<OdrsReviewsBackend> m_reviews;
     uint m_isFetching = 0;
-    uint m_refreshAppstreamMetadataJobs;
+    QSet<FlatpakRemote *> m_refreshAppstreamMetadataJobs;
     QStringList m_extends;
 
     GCancellable *m_cancellable;
@@ -134,6 +147,5 @@ private:
     QVector<QSharedPointer<FlatpakSource>> m_flatpakSources;
     QVector<QSharedPointer<FlatpakSource>> m_flatpakLoadingSources;
     QSharedPointer<FlatpakSource> m_localSource;
+    QTimer *const m_checkForUpdatesTimer;
 };
-
-#endif // FLATPAKBACKEND_H
